#include "mbed.h"
#include "nRF24L01P.h"
#include "beep.h"

#define nrf_CE      D2
#define nrf_CSN     A3
#define spi_SCK     D13
#define spi_MOSI    D11
#define spi_MISO    D12
#define spi_IRQ     D4

#define NRF_TRANSMIT    1
#define NRF_RECEIVE     2

#define NRF_ID  7

#define TRANSFER_SIZE 15

nRF24L01P nrf(spi_MOSI, spi_MISO, spi_SCK, nrf_CSN, nrf_CE, spi_IRQ);    // mosi, miso, sck, csn, ce, irq

uint64_t rxAddr=0xDEADBEEF0F;
uint64_t txAddr=0xDEADBEEF00;

Serial pc(USBTX, USBRX);
//Serial lidar(D1, D0);

PwmOut motor_RA(D9);
PwmOut motor_RB(D10);
PwmOut motor_LA(D3);
PwmOut motor_LB(D6);
PwmOut led_B(A5);
PwmOut led_G(A2);
PwmOut led_R(A1);
PwmOut buzzer(D5);
AnalogIn batteryCheck(A0);

void beepStart();
void dumpRFInfo();
void turnWheel(int rspd, int lspd);
void initNRF(int mode, int msg_len);

int main()
{
    char rxData[TRANSFER_SIZE];
    int rxDataCnt = 0;

    initNRF(NRF_RECEIVE, TRANSFER_SIZE);
    int inByteCnt = 0;

    beepStart();
    pc.baud(115200);
//    dumpRFInfo();
    led_G = 1;
    led_R = 1;

    while(1) {
        if ( nrf.readable() ) {
            rxDataCnt = nrf.read( NRF24L01P_PIPE_P0, rxData, TRANSFER_SIZE);
            if(rxData[0] =='$') {
                int id = (rxData[1]-'0')*10+(rxData[2]-'0');
                int dataCount = (rxData[3]-'0')*10000+(rxData[4]-'0')*1000+(rxData[5]-'0')*100+(rxData[6]-'0')*10+(rxData[7]-'0');
                int rspd = (rxData[9]-'0')*10+(rxData[10]-'0');
                int lspd = (rxData[12]-'0')*10+(rxData[13]-'0');
                if(rxData[8]=='-')
                    rspd = -rspd;
                if(rxData[11]=='-')
                    lspd = -lspd;

                if(id==NRF_ID) {
                    turnWheel(rspd, lspd);
                    printf("[%d] RxData: %s, rspd: %d, rspd: %d\r\n", id, rxData, rspd, lspd);
                    char buf[] = "JUSTDUMMYREPLY";
                    wait(0.005);
                    nrf.setTxAddress(0xDEADBEEF00);
                    nrf.setTransferSize( TRANSFER_SIZE );
                    nrf.setTransmitMode();
                    nrf.enable();
                    nrf.write(NRF24L01P_PIPE_P0, buf, TRANSFER_SIZE);

                    nrf.setRxAddress(0xDEADBEEF0F);
                    nrf.powerUp();
                    nrf.setTransferSize( TRANSFER_SIZE );
                    nrf.setReceiveMode();
                    nrf.enable();
                }
            } else {
                printf("Invalid Data\r\n");
            }
        }
    }
}


void turnWheel(int rspd, int lspd)
{
    if(rspd>0) {
        motor_RA.write((float)rspd/100);
        motor_RB = 0;
    } else {
        rspd=-rspd;
        motor_RB.write((float)rspd/100);
        motor_RA = 0;
    }
    if(lspd>0) {
        motor_LA.write((float)lspd/100);
        motor_LB = 0;
    } else {
        lspd = -lspd;
        motor_LB.write((float)lspd/100);
        motor_LA = 0;
    }
}

void dumpRFInfo()
{
    printf( "nRF24L01+ Frequency    : %d MHz\r\n",  nrf.getRfFrequency() );
    printf( "nRF24L01+ Output power : %d dBm\r\n",  nrf.getRfOutputPower() );
    printf( "nRF24L01+ Data Rate    : %d kbps\r\n", nrf.getAirDataRate() );
    printf( "nRF24L01+ TX Address   : 0x%010llX\r\n", nrf.getTxAddress() );
    printf( "nRF24L01+ RX Address   : 0x%010llX\r\n", nrf.getRxAddress() );
}

void initNRF(int mode, int msg_len)
{
#ifndef NRF_FIRST_INIT
#define NRF_FIRST_INIT
    nrf.setTxAddress(txAddr);
    nrf.setRxAddress(rxAddr);
    nrf.powerUp();
#endif
    switch(mode) {
        case NRF_TRANSMIT:
            nrf.setTransferSize(msg_len);
            nrf.setTransmitMode();
            nrf.enable();
            break;
        case NRF_RECEIVE:
            nrf.setTransferSize(msg_len);
            nrf.setReceiveMode();
            nrf.enable();
            break;
        default:
            printf("Invalid NRF Mode\r\n");
    }
}


/*
xxyyyyyabbcddn

14 chars

xx: robot_id
yyyyy: packet_id
a: sign of lspeed
bb: lspeed
c: sign of rspeed
dd: rspeed
n: NULL 0

0100001+30+30
*/